"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = require("path");

var _fs = require("fs");

var _requireGql = require("./requireGql");

var resolve;

var _default = function _default(_ref) {
  var t = _ref.types,
      template = _ref.template;
  return {
    manipulateOptions(_ref2) {
      var resolveModuleSource = _ref2.resolveModuleSource;

      if (!resolve) {
        resolve = resolveModuleSource || _requireGql.defaultResolve;
      }
    },

    visitor: {
      ImportDeclaration: {
        exit(curPath, _ref3) {
          var opts = _ref3.opts,
              file = _ref3.file;
          var importPath = curPath.node.source.value;

          if (importPath.endsWith('.graphql') || importPath.endsWith('.gql')) {
            // Find the file, using node resolution/NODE_PATH if necessary.
            var fallbackPaths = opts.nodePath ? opts.nodePath.split(_path.delimiter) : process.env.NODE_PATH;
            var absPath = resolve(importPath, file.opts.filename);
            if (!(0, _fs.existsSync)(absPath)) absPath = require.resolve(importPath, {
              paths: fallbackPaths
            }); // Analyze the file, returning either schema source (str) or a docs object containing ops.

            var result = (0, _requireGql.requireGql)(absPath, {
              resolve,
              nowrap: false
            });
            var importNames = curPath.node.specifiers;

            if (typeof result === 'string') {
              curPath.replaceWith(buildVariableAST(result, importNames[0].local.name));
            } else {
              var replacements = buildReplacements(result, importNames);
              replacements.length > 1 ? curPath.replaceWithMultiple(replacements) : curPath.replaceWith(replacements[0]);
            }
          }

          function buildReplacements(docs, specifiers) {
            return specifiers.map(function (_ref4) {
              var type = _ref4.type,
                  imported = _ref4.imported,
                  local = _ref4.local;

              switch (type) {
                case 'ImportDefaultSpecifier':
                  return buildVariableAST(docs.default, local.name);

                case 'ImportSpecifier':
                  return buildVariableAST(docs[imported.name] || docs.default, local.name);

                case 'ImportNamespaceSpecifier':
                  return buildVariableAST(docs, local.name);

                default:
                  throw new Error(`Unexpected import specifier type: ${type}`);
              }
            });
          }

          function buildVariableAST(graphqlAST, importName) {
            var buildAst = template(`var IMPORT_NAME = AST;`);
            return buildAst({
              IMPORT_NAME: t.identifier(importName),
              AST: t.valueToNode(JSON.parse(JSON.stringify(graphqlAST)))
            });
          }
        }

      }
    }
  };
};

exports.default = _default;